var fillImage;
var sensorUnit;
var maxValue;
var minValue;
var rpmMax = 0;
var signalMax = 0;
var setpoints;
var engineOnOnly = 0;
var rpmSignal = true;

function body_onload() {
    getSetpoints();
}

function draw() {
    var canvas = document.getElementById("rpm_canvas");
    if (null==canvas || !canvas.getContext) return;

    getDcuData();
}

function getSetpoints() {
    $.ajax({
        type: "GET", url: "/RpmSetpoints",
        success: function (doc, text, data) {
            var parsed = $.parseJSON(data.responseText)
            setpoints = parsed.setpoints;
        },
        error: function (request, status, error) {
        },
        complete: function (request, status) {
            draw();
        }
    });
}

function getChannelConfig(id) {
    $.ajax({
        type: "GET", url: "/GetChannelConfig?id=" + id + "&unit=" + sensorUnit,
        success: function (doc, text, data) {
            var channelConfig = $.parseJSON(data.responseText);
            var enabled = channelConfig.channel_enabled;
            rpmMax = channelConfig.display.instrument_range_max;
            if((enabled > 0) && (rpmMax > 0)) {
                rpmMax = ceil(rpmMax/3.5)*3.5;
            } else {
                rpmMax = 0;
            }
        },
        error: function (request, status, error) {
            rpmMax = 0;
        },
        complete: function (request, status) {
            signalMax = Math.max(rpmMax, signalMax);
            if(signalMax <= 0) {
                signalMax = 3150;
            }
            drawChart();
        }
    });
}

function getDcuData()
{
    $.ajax({
        type: "GET", url: "/SensorUnit",
        success: function (doc, text, data) {
            sensorUnit = data.responseText;

            var maxValueElem = document.getElementById("instrument_range_max_" + sensorUnit);
            var minValueElem = document.getElementById("instrument_range_min_" + sensorUnit);
            var alarmSignalElem = document.getElementById("alarm_dependent_signal");
            var min = 0;
            var max = 0;
            var diff;
            var diff_ceil;
            var points = getPoints();
            if(maxValueElem && minValueElem) {
                maxValue = Math.max(parseFloat(maxValueElem.value), 0);
                minValue = Math.min(parseFloat(minValueElem.value), 0);
            } else {
                maxValue = 0;
                minValue = 0;
            }
            for(var i=0; i<points.length; i++) {
                min = Math.min(minValue, min, points[i].limit_low, points[i].limit_high);
                max = Math.max(maxValue, max, points[i].limit_low, points[i].limit_high);
            }
            if((maxValue == minValue) || (max > maxValue) || (min < minValue)) {
                min *= 1.05;
                max *= 1.05;
                diff = max - min;
                diff_ceil = ceil(diff*2.5)/2.5;
                if(min < 0) {
                    min -= (diff_ceil - (diff))/2;
                } else {
                    max += (diff_ceil - (diff))/2;
                }
                if(max > 0) {
                    max += (diff_ceil - (diff))/2;
                } else {
                    min -= (diff_ceil - (diff))/2;
                }
                maxValue = max;
                minValue = min;
            }

            var engineOnOnlyElem = document.getElementById("alarm_engine_on_only");
            if(engineOnOnlyElem) {
                engineOnOnly = parseInt(engineOnOnlyElem.value);
            }

            signalMax = 0;
            var points = getPoints();
            for(var i=0; i<points.length; i++) {
                signalMax = Math.max(signalMax, points[i].rpm);
            }

            if(alarmSignalElem) {
                if(parseInt(alarmSignalElem.value) == 2077) {
                    rpmSignal = false;
                    signalMax = Math.max(signalMax, 100);
                } else {
                    rpmSignal = true;
                }
            } else {
                rpmSignal = true;
            }
            signalMax = ceil(signalMax/3.5)*3.5;
            if(rpmSignal) {
                if(rpmMax == 0) {
                    getChannelConfig(2049);
                } else {
                    signalMax = Math.max(rpmMax, signalMax);
                    drawChart();
                }
            } else {
                drawChart();
            }
        },
        error: function (request, status, error) {
        }
    });
}

function rpmSetpoint() {
    if(!rpmSignal) {
        var engineLoadElem = document.getElementById("alarm_engine_load");
        if(engineLoadElem) {
            var engineLoad = parseInt(engineLoadElem.value);
            if(engineLoad) {
                engineLoadElem = document.getElementById("alarm_engine_load_threshold");
                if(engineLoadElem) {
                    engineLoad = parseInt(engineLoadElem.value);
                    if(engineLoad) {
                        return engineLoad;
                    }
                }
            }
        }
        return 0;
    }
    if(!setpoints) {
        return 0;
    }
    switch(engineOnOnly) {
        case 0:
        return 0;

        case 1:
        return setpoints[0];

        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
        return setpoints[engineOnOnly - 2];

        default:
        return 0;
    }
}

function drawChart() {
    var canvas = document.getElementById("rpm_canvas");
    var ctx=canvas.getContext("2d");

    ctx.clearRect ( 0 , 0 , canvas.width , canvas.height);

    fillImage = new Image();
    fillImage.onload = function() {
        var alarm_high = parseInt(document.getElementById("alarm_direction").value);
        var points = getPoints(alarm_high);
        var yLevels = valueLevels(minValue, maxValue);
        
        drawGrid(ctx, yLevels);
        if(maxValue - minValue > 0) {
            if(alarm_high == 2) {
                drawLimitCurve(ctx, points, 0);
                drawLimitCurve(ctx, points, 1);
            } else {
                drawLimitCurve(ctx, points, alarm_high);
            }
        }
        drawBorder(ctx)
        drawRpmLabels(ctx);
        if(maxValue - minValue > 0) {
            drawValueLabels(ctx, yLevels);
        }
    }
    fillImage.src = "fill.png";
}

function getPoints(alarm_high) {
    var points = parseInt(document.getElementById("alarm_rpm_points").value);
    var curvePoints = new Array();

    var minRpm = 0;

    for(var i = 0; i < points; ++i) {
        var limit_low = 0;
        var limit_high = 0;
        var rpm_id = "alarm_rpm_rpm_" + (i + 1);
        if(alarm_high == 0) {
            var limit_id = "alarm_rpm_limit_" + (i + 1);
            limit_low = parseFloat(document.getElementById(limit_id).value);
        } else if(alarm_high == 1) {
            var limit_id = "alarm_rpm_limit_" + (i + 1);
            limit_high = parseFloat(document.getElementById(limit_id).value);
        } else {
            var limit_high_id = "alarm_rpm_limit_" + (i + 1);
            var limit_low_id = "alarm_rpm_limit2_" + (i + 1);
            limit_high = parseFloat(document.getElementById(limit_high_id).value);
            limit_low = parseFloat(document.getElementById(limit_low_id).value);
        }
        var rpm = parseFloat(document.getElementById(rpm_id).value);
        var point = {};
        point.rpm = rpm;
        point.limit_low = limit_low;
        point.limit_high = limit_high;
        point.index = i;
        curvePoints[i] = point;
    }

    curvePoints.sort(function(a,b) {
        if(a.rpm == b.rpm) {
            return a.index - b.index;
        } else {
            return a.rpm - b.rpm
        }
    });
    
    return curvePoints;
}

function drawBorder(ctx) {
    ctx.lineWidth = 1;
    ctx.strokeStyle = "rgb(0,0,0)";
    ctx.strokeRect(0,0,ctx.canvas.width,ctx.canvas.height); 
}

function drawGrid(ctx, yLevels) {
    var width = ctx.canvas.width;
    var height = ctx.canvas.height;
    var ySteps = 5;

    if(yLevels.length > 0) {
        ySteps = yLevels.length;
    }

    ctx.beginPath();
    ctx.lineWidth = 1;
    ctx.strokeStyle = "rgb(128,128,128)";

    for(var x = 1; x < 14; ++x) {
        ctx.moveTo(0.5 + Math.round(width/14) * x , 0);
        ctx.lineTo(0.5 + Math.round(width/14) * x , height);
    }

    for(var y = 0; y < ySteps; y++) {
        var point = {};
        if(yLevels.length > 0) {
            point = convertToCanvasPoint(ctx, 0, yLevels[y], 0, signalMax, minValue, maxValue);
        } else {
            point.y = (height/ySteps)*(y + 1) - 0.5;
        }
        ctx.moveTo(0, point.y);
        ctx.lineTo(width, point.y);
    }

    ctx.stroke();
}

function drawRpmLabels(ctx) {
    var width = ctx.canvas.width;
    var height = ctx.canvas.height;

    ctx.font="12px Georgia";
    for(var i = 2; i < 14; i+=2) {
        var text = ((i/2) * Math.round(signalMax/7)).toString();
        var text_width = ctx.measureText(text).width;
        var x = (width/14) * i - text_width/2;
        var y = height - 1;
        ctx.fillText(text, x, y);
    }
}

function drawValueLabels(ctx, yValues) {
    ctx.font="12px Georgia";
    for(var i = 0; i < yValues.length; i++) {
        var point = convertToCanvasPoint(ctx, 0, yValues[i], 0, signalMax, minValue, maxValue);
        point.x = 1;
        point.y += 12;
        ctx.fillText(yValues[i], point.x, point.y);
    }
}

function valueLevels(yMin, yMax) {
    var decimals = 0;
    var step = (yMax-yMin)/5;
    var levels = [];

    if(maxValue - minValue <= 0) {
        return levels;
    }

    if(step < 100) {
        decimals=1;
        if(step < 10) {
            decimals=2;
        }
    }

    for(var y = 0; y <= yMax; y+=step) {
        var pow = Math.pow(10,decimals);
        levels.push(Math.round(pow*y)/pow);
    }
    for(var y = -step; y >= yMin; y-=step) {
        var pow = Math.pow(10,decimals);
        levels.push(Math.round(pow*y)/pow);
    }
    return levels;
}

function limitAtRpm(points, rpm) {
    var rv = new Array();
    if(rpm <= points[0].rpm) {
        rv.low = points[0].limit_low;
        rv.high = points[0].limit_high;
    } else if(rpm >= points[points.length-1].rpm) {
        rv.low = points[points.length-1].limit_low;
        rv.high = points[points.length-1].limit_high;
    } else {
        for(var i=1;i<points.length;i++) {
            if(rpm <= points[i].rpm) {
                var k = (points[i].limit_low - points[i-1].limit_low) /
                    (points[i].rpm - points[i-1].rpm);
                var b = (points[i-1].limit_low) - k*points[i-1].rpm;
                rv.low = (k * rpm + b);

                k = (points[i].limit_high - points[i-1].limit_high) /
                    (points[i].rpm - points[i-1].rpm);
                b = (points[i-1].limit_high) - k*points[i-1].rpm;
                rv.high = (k * rpm + b);
                return rv;
            }
        }
    }
    return rv;
}

function drawLimitCurve(ctx, points, alarm_high) {
    var width = ctx.canvas.width;
    var height = ctx.canvas.height;
    var limit;
    if(alarm_high != 0) {
        limit = limitAtRpm(points, rpmSetpoint()).high;
    } else {
        limit = limitAtRpm(points, rpmSetpoint()).low;
    }
    var point = convertToCanvasPoint(ctx, 
                                     rpmSetpoint(), limit, 
                                     0, signalMax, minValue, maxValue);

    ctx.beginPath();
    ctx.lineWidth = 2;
    ctx.strokeStyle = "rgb(255,0,0)";    
    
    if(alarm_high != 0) {
        ctx.moveTo(point.x, 0);
    } else {
        ctx.moveTo(point.x, height);
    }

    ctx.lineTo(point.x, point.y);

    var rpm = rpmSetpoint();
    for(var i = 0; i < points.length; ++i) {
        if(points[i].rpm >= rpm) {
            if(alarm_high != 0) {
                limit = points[i].limit_high;
            } else {
                limit = points[i].limit_low;
            }
            rpm = points[i].rpm;
            point = convertToCanvasPoint(ctx, points[i].rpm, limit, 
                                         0, signalMax, minValue, maxValue);
            ctx.lineTo(point.x, point.y);
        }
    }

    if(alarm_high != 0) {
        limit = points[points.length-1].limit_high;
    } else {
        limit = points[points.length-1].limit_low;
    }
    point = convertToCanvasPoint(ctx, 
                                 signalMax, limit,
                                 0, signalMax, minValue, maxValue);
    ctx.lineTo(point.x, point.y);

    if(alarm_high != 0) {
        point = convertToCanvasPoint(ctx, signalMax, maxValue, 0, signalMax, minValue, maxValue);
    } else {
        point = convertToCanvasPoint(ctx, signalMax, minValue, 0, signalMax, minValue, maxValue);
    }
    ctx.lineTo(point.x, point.y);

    ctx.stroke();
    ctx.closePath();

    ctx.save();
    var pat=ctx.createPattern(fillImage,"repeat");
    ctx.fillStyle=pat;
    ctx.fill();
    ctx.restore();
}

function drawSegment(ctx, rpm0, value0, rpm1, value1) {
    var point = convertToCanvasPoint(ctx, rpm0, value0, 0, signalMax, minValue, maxValue);
    ctx.moveTo(point.x, point.y);
    point = convertToCanvasPoint(ctx, rpm1, value1, 0, signalMax, minValue, maxValue);
    ctx.lineTo(point.x, point.y);
}

function convertToCanvasPoint(ctx, x, y, xMin, xMax, yMin, yMax) {
    var width = ctx.canvas.width;
    var height = ctx.canvas.height;

    var point = {};

    point.x = ((x - xMin)/(xMax - xMin))*width;
    point.y = height - ((y - yMin)/(yMax - yMin))*height;

    return point;
}

function ceil(n) {
    if(n == 0) {
        return 0;
    } else if(n < 0) {
        return -ceil(-n);
    }
    var m = Math.ceil(Math.log(n)/Math.LN10);
    m = Math.pow(10, 1 - m);
    return Math.ceil(n*m)/m;
}
